#include <gtk/gtk.h>
#include <gst/gst.h>
#include <glib.h>
#include <libxml/parser.h>
#include <libxml/tree.h>
#include "gnome-voice-stream.h"

#define GNOME_VOICE_XML "GNOME.voice"

typedef struct {
    char *url;
    char *location;
    double latitude;
    double longitude;
    GtkWidget *play_button;
    GtkWidget *stop_button;
    GstElement *pipeline;  // GStreamer pipeline for each stream
} Stream;

GList *streams = NULL;
GtkWidget *url_entry;
GtkWidget *main_box;

// Function to save the stream list to XML
void save_streams_to_xml() {
    xmlDocPtr doc = NULL;
    xmlNodePtr root_node = NULL;

    // Create a new XML document and root node
    doc = xmlNewDoc(BAD_CAST "1.0");
    root_node = xmlNewNode(NULL, BAD_CAST "voice");
    xmlDocSetRootElement(doc, root_node);
    xmlNewProp(root_node, BAD_CAST "version", BAD_CAST "1.2.0");

    GList *l;
    for (l = streams; l != NULL; l = l->next) {
        Stream *stream = (Stream *)l->data;

        // Create a "station" node
        xmlNodePtr station_node = xmlNewChild(root_node, NULL, BAD_CAST "station", NULL);
        xmlNewProp(station_node, BAD_CAST "uri", BAD_CAST stream->url);

        // Create the location node
        xmlNodePtr location_node = xmlNewChild(station_node, NULL, BAD_CAST "location", BAD_CAST stream->location);
        xmlNewProp(location_node, BAD_CAST "lat", BAD_CAST g_strdup_printf("%f", stream->latitude));
        xmlNewProp(location_node, BAD_CAST "lon", BAD_CAST g_strdup_printf("%f", stream->longitude));

        // Create the stream node
        xmlNodePtr stream_node = xmlNewChild(station_node, NULL, BAD_CAST "stream", BAD_CAST stream->url);
    }

    // Save the document to the file
    xmlSaveFormatFileEnc(GNOME_VOICE_XML, doc, "UTF-8", 1);
    xmlFreeDoc(doc);
}

// Function to load streams from XML
void load_streams_from_xml() {
    xmlDocPtr doc;
    xmlNodePtr root_element, station_node, location_node, stream_node;
    char *url, *location;
    double lat, lon;

    // Parse the XML file
    doc = xmlReadFile(GNOME_VOICE_XML, NULL, 0);
    if (doc == NULL) {
        g_print("Failed to parse the XML file\n");
        return;
    }

    root_element = xmlDocGetRootElement(doc);
    station_node = root_element->children;

    // Iterate over each "station" element in the XML
    for ( ; station_node != NULL; station_node = station_node->next) {
        if (station_node->type == XML_ELEMENT_NODE && xmlStrcmp(station_node->name, BAD_CAST "station") == 0) {
            url = (char *)xmlGetProp(station_node, BAD_CAST "uri");
            location_node = station_node->children;
            location = (char *)xmlNodeGetContent(location_node);

            lat = atof((char *)xmlGetProp(location_node, BAD_CAST "lat"));
            lon = atof((char *)xmlGetProp(location_node, BAD_CAST "lon"));

            Stream *new_stream = g_new(Stream, 1);
            new_stream->url = g_strdup(url);
            new_stream->location = g_strdup(location);
            new_stream->latitude = lat;
            new_stream->longitude = lon;

            // Create buttons for play, stop, and record for this stream
            new_stream->play_button = gtk_button_new_with_label("Play");
            g_signal_connect(new_stream->play_button, "clicked", G_CALLBACK(play_stream), new_stream);

            new_stream->stop_button = gtk_button_new_with_label("Stop");
            g_signal_connect(new_stream->stop_button, "clicked", G_CALLBACK(stop_stream), new_stream);

            // Add to streams list
            streams = g_list_append(streams, new_stream);

            // Create a new hbox for this stream's buttons
            GtkWidget *hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 5);
            gtk_box_pack_start(GTK_BOX(hbox), new_stream->play_button, FALSE, FALSE, 0);
            gtk_box_pack_start(GTK_BOX(hbox), new_stream->stop_button, FALSE, FALSE, 0);

            // Add the hbox to the main window (or a container you prefer)
            gtk_box_pack_start(GTK_BOX(main_box), hbox, FALSE, FALSE, 5);
        }
    }

    xmlFreeDoc(doc);
}

// Function to play a stream
void play_stream(GtkWidget *button, gpointer data) {
    Stream *stream = (Stream *)data;
    g_print("Playing stream from URL: %s\n", stream->url);

    // Initialize GStreamer
    if (!gst_is_initialized()) {
        gst_init(NULL, NULL);
    }

    // Create a pipeline for the stream
    stream->pipeline = gst_parse_launch(g_strdup_printf("playbin uri=%s", stream->url), NULL);
    
    // Start playing the stream
    gst_element_set_state(stream->pipeline, GST_STATE_PLAYING);
}

// Function to stop a stream
void stop_stream(GtkWidget *button, gpointer data) {
    Stream *stream = (Stream *)data;
    g_print("Stopping stream from URL: %s\n", stream->url);

    if (stream->pipeline) {
        // Stop the GStreamer pipeline
        gst_element_set_state(stream->pipeline, GST_STATE_NULL);
        gst_object_unref(stream->pipeline);
        stream->pipeline = NULL;
    }
}

// Function to add a stream
void add_stream(GtkWidget *button, gpointer data) {
    const char *url = gtk_entry_get_text(GTK_ENTRY(url_entry));
    if (url == NULL || *url == '\0') {
        g_print("URL input is empty\n");
        return;
    }

    Stream *new_stream = g_new(Stream, 1);
    new_stream->url = g_strdup(url);
    new_stream->location = g_strdup("Unknown Location");
    new_stream->latitude = 0.0;
    new_stream->longitude = 0.0;

    // Create buttons for play, stop, and record for this stream
    new_stream->play_button = gtk_button_new_with_label("Play");
    g_signal_connect(new_stream->play_button, "clicked", G_CALLBACK(play_stream), new_stream);

    new_stream->stop_button = gtk_button_new_with_label("Stop");
    g_signal_connect(new_stream->stop_button, "clicked", G_CALLBACK(stop_stream), new_stream);

    // Add to streams list
    streams = g_list_append(streams, new_stream);

    // Create a new hbox for this stream's buttons
    GtkWidget *hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 5);
    gtk_box_pack_start(GTK_BOX(hbox), new_stream->play_button, FALSE, FALSE, 0);
    gtk_box_pack_start(GTK_BOX(hbox), new_stream->stop_button, FALSE, FALSE, 0);

    // Add the hbox to the main window (or a container you prefer)
    gtk_box_pack_start(GTK_BOX(main_box), hbox, FALSE, FALSE, 5);

    gtk_widget_show_all(main_box);

    // Save the new stream to the XML file
    save_streams_to_xml();
}

// Function to delete a stream
void delete_stream(GtkWidget *button, gpointer data) {
    Stream *stream = (Stream *)data;
    g_print("Deleting stream from URL: %s\n", stream->url);
    
    // Remove the stream from the list
    streams = g_list_remove(streams, stream);
    
    // Free memory
    g_free(stream->url);
    g_free(stream->location);
    g_free(stream);

    // Save updated streams to the XML file
    save_streams_to_xml();
}

int main(int argc, char *argv[]) {
    GtkWidget *window, *vbox, *url_label, *add_button;

    gtk_init(&argc, &argv);

    window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title(GTK_WINDOW(window), "GNOME Voice");
    gtk_window_set_default_size(GTK_WINDOW(window), 800, 600);

    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 10);
    gtk_container_add(GTK_CONTAINER(window), vbox);

    // URL input field and label
    url_label = gtk_label_new("Enter Stream URL:");
    gtk_box_pack_start(GTK_BOX(vbox), url_label, FALSE, FALSE, 0);

    url_entry = gtk_entry_new();
    gtk_box_pack_start(GTK_BOX(vbox), url_entry, FALSE, FALSE, 0);

    // Add button to add stream
    add_button = gtk_button_new_with_label("Add Stream");
    g_signal_connect(add_button, "clicked", G_CALLBACK(add_stream), NULL);
    gtk_box_pack_start(GTK_BOX(vbox), add_button, FALSE, FALSE, 0);

    // Main box for stream buttons
    main_box = gtk_box_new(GTK_ORIENTATION_VERTICAL, 5);
    gtk_box_pack_start(GTK_BOX(vbox), main_box, TRUE, TRUE, 0);

    g_signal_connect(window, "destroy", G_CALLBACK(gtk_main_quit), NULL);
    gtk_widget_show_all(window);

    // Load previously saved streams from XML
    load_streams_from_xml();

    gtk_main();

    return 0;
}
